@@ -18,14 +18,14 @@ sub MonitisMonitorManager::_get_PGSQL_DB_NAME {
 
 sub MonitisMonitorManager::_get_API_KEY {
 	# remove croak line after you replace the XXX with your API key
-	croak "Please edit M3Templates.pm with your proper API key";
-	return "XXX";
+	#croak "Please edit M3Templates.pm with your proper API key";
+	return "7A04S52D7FOOS5IC5CPMJEJ0N2";
 }
 
 sub MonitisMonitorManager::_get_SECRET_KEY {
 	# remove croak line after you replace the XXX with your secret key
-	croak "Please edit M3Templates.pm with your proper secret key";
-	return "XXX";
+	#croak "Please edit M3Templates.pm with your proper secret key";
+	return "7RLFT91MQIBVH024UT6ELH611T";
 }
 
 1;
